body.fullscreen #home .info {
    display: none;
}

html {
	background: #000;
	color: #fff;
	margin: 0;
	padding: 0;
}

body {
	position: relative;
	background: #000;
	color: #fff;
	width: 314px;
	margin: 0;
	padding: 0 0 50px 0;

	font-size: 16px;
	font-family: sans-serif;
	line-height: 1.66;
}

a:link {
	color: #cccccc;
	text-decoration: none;
}

.even {
	background: -webkit-gradient(linear, 0% 0%, 0% 100%, from(#222222), to(#222222), color-stop(.6,#333333));
	color: #fff;
}

.odd {
	background: #000;
	color: #fff;
}

#ChannelList {
    padding-bottom: 30px;
}

ol.channels {
	margin: 0;
	padding: 0;
}

ol.channels li {
	margin: 0;
	padding: 0;
}

ol.channels li a {
	display: block;
	position: relative;
	font-weight: bold;
	font-size: 20px;
	padding: 15px;
	color: #fff;
	text-shadow: 0 0 5px #333;
	white-space: nowrap;
}

ol.channels li a span.unread {
	position: absolute;
	right: 20px;
	top: 15px;
}

form div.input {
	padding: 5px;
	margin: 0 30px 0 0;
}

form div.input input.text {
	width: 100%;
}

form select.post {
	position: absolute;
	top: 0;
	right: 5px;
	width: 25px;
	height: 24px;
	margin: 7px 0;;
	padding: 0 5px;
	outline: none;
	border: 1px solid #000;
	background: #999;
	color: #fff;
	text-align: center;
	-webkit-appearance: none;
}


div.message {
	position: relative;
	font-weight: bold;
	font-size: 13px;
	padding: 5px 10px;
	background: -webkit-gradient(linear, 0% 0%, 0% 100%, from(#222222), to(#000000), color-stop(.6,#111111));
	color: #fff;
}

div.message span.time {
	display: block;
	position: absolute;
	top: 0;
	right: 10px;
	font-weight: bold;
	padding: 5px 0;
	margin: 0;
	color: #fff;
	text-shadow: 0 0 5px #999;
}

div.message div.body {
	margin: 0 50px 0 0;
    word-break: break-all;
}

div.message div.body img {
	display: block;
	margin: 5px auto;
}


div.message.join ,
div.message.leave {
}

div.message.notice,
div.message.public {
	padding: 10px;
	background: #000;
	color: #fff;
}

div.message.notice span.who,
div.message.public span.who {
	display: block;
	font-weight: bold;
	margin: -10px -10px 0 -10px;
	padding: 5px 10px;
	background: -webkit-gradient(linear, 0% 0%, 0% 100%, from(#444444), to(#222222), color-stop(.6,#333333));
	color: #ffffff;
	text-shadow: 0 0 5px #333;
}

div.message.notice.new span.who,
div.message.public.new span.who {
	background: -webkit-gradient(linear, 0% 0%, 0% 100%, from(#969696), to(#828282), color-stop(.6,#636363));
	color: #fff;
}

div.message.notice img.avatar ,
div.message.public img.avatar {
	float: right;
	margin: 5px 0 5px 10px;
	height: 32px;
}


div.message.notice span.time,
div.message.public span.time {
	color: #dddddd;
}

div.message.public div.body ,
div.message.notice div.body {
	min-height: 32px;
	margin: 5px 0 0 0;
}

div.message.notice {
    color: #777777;
    font-style: italic;
}

select {
}

select.operations {
	margin: 0;
	padding: 0;
	border: none;
	outline: none;
	background: transparent;
	color: #ccc;
	-webkit-appearance: button;
}

img.loading {
	position: absolute;
	top: 12px;
	right: 10px;
}

.pager {

}

.pager a {
	text-align: right;
	padding: 10px;
	display: block;
	font-weight: bold;
	background: -webkit-gradient(linear, 0% 0%, 0% 100%, from(#111111), to(#111111), color-stop(.6,#222222));
	text-decoration: none;
	color: #fff;
}

#MiscMenuContainer {
    margin-left: 8px;
    margin-top: 4px;
}

#RefreshMenuContainer {
    margin: 5px;
}

footer {
    border-top: #cccccc 1px solid;
    margin-top: 10px;
    text-align: right;
    padding-right: 10px;
}
