/* page layout */

#nav { background-color: #2B2E2E }

#BodyWrapper { }
#PageBody { overflow: hidden; clear: both; } 
#PageBody #Main { overflow: hidden; } 
#PageBody #Main #ChannelPane { overflow: hidden; }
#PageBody #Main #ChannelLog { overflow: auto; width: 100%; font-size: 120%; } 
#PageBody #Main #CommandForm { width: 100%;  margin-top: 10px; margin-bottom: 20px; }
#PageBody #Main #CommandForm #msg { width: 90%; }
#PageBody #Main #CommandForm #SendButton { width: 8%; }
#PageBody #Main #CombinedLog { overflow: auto; width: 100%; margin-top: 4px; } 
#PageBody #Side { overflow: hidden; height: 100%; padding-left: 5px; }
#PageBody #Side #NickContainer { overflow: auto; border-bottom: 1px solid #cccccc; cursor: default; -moz-user-select: none; user-select: none; -webkit-user-select: none; }
#PageBody #Side #ChannelContainer { overflow: auto; cursor: default; -moz-user-select: none; user-select: none; -webkit-user-select: none; }

#PageBody #Side .channel { margin-bottom: 2px; width: 100%; }
#PageBody #Side .unread  { background-color: #99ffff; }
#PageBody #Side .current { background-color: #CAE6E8; }
#PageBody #Side .keyword { background-color: #ff99ff; }

.log .time { margin-right: 2px; }
.log .message { margin-left: 2px; }
.log .channel { margin-left: 2px; margin-right: 2px; }
.log:nth-child(even) { background-color: #eeeeee; }

.ui-layout-pane {
    background-color: #fff !important;
}
.ui-layout-resizer {
    background-color: #ccc !important;
}

